import zmq 
import time

from .. import _gondola_core  as _gc 

class TelemetrySocketReader:

    def __init__(self, address):
        ctx  = zmq.Context()
        sock = ctx.socket(zmq.SUB)
        sock.subscribe("")
        sock.connect(address)
        self.socket = sock 
    
    def __iter__(self):
        return self  # Return self, as this class is also the iterator

    def __next__(self):
        data  = self.socket.recv()
        pack  = _gc.packets.TelemetryPacket.from_bytestream(data, 0)
        frame = _gc.io.CRFrame()
        frame.put_telemetrypacket(pack) 
        return frame 


class PacketStreamer:

    def __init__(self, reader, address, rate = 500):
        self.reader  = reader 
        ctx          = zmq.Context()
        self.socket  = ctx.socket(zmq.PUB)
        self.socket.bind(address)
        self.rate    = rate 

    def stream(self):
        """
        This will block!
        """
        while True:
            pack = self.reader.__next__()
            if pack is None:
                self.reader.rewind()
                continue 
            self.socket.send(pack.to_bytestream())
            time.sleep(1/self.rate)



"""
Every possible event type in GAPS which is relevant for 
"online" analysis 
"""

from . import _gondola_core 

TofHit                   =  _gondola_core.events.TofHit             
TofHit.__module__        = __name__
TofHit.__name__          = 'TofHit'
TrackerHit               =  _gondola_core.events.TrackerHit         
TrackerHit.__module__    = __name__ 
TrackerHit.__name__      = 'TrackerHit'
RBEventHeader            =  _gondola_core.events.RBEventHeader      
RBEventHeader.__module__ = __name__ 
RBEventHeader.__name__   = 'RBEventHeader' 
RBEvent                  =  _gondola_core.events.RBEvent    
RBEvent.__module__       = __name__ 
RBEvent.__name__         = 'RBEvent'
RBWaveform               =  _gondola_core.events.RBWaveform    
RBWaveform.__module__    = __name__
RBWaveform.__name__      = 'RBWaveform'

TofEvent          =  _gondola_core.events.TofEvent           
TofEvent.__module__ = __name__ 
TofEvent.__name__ = 'TofEvent' 
TelemetryEvent    =  _gondola_core.events.TelemetryEvent     
TelemetryEvent.__module__ = __name__ 
TelemetryEvent.__name__ = 'TelemetryEvent' 
# functions 
strip_id          =  _gondola_core.events.strip_id           
# enums
EventQuality      =  _gondola_core.events.EventQuality       
TriggerType       =  _gondola_core.events.TriggerType        
LTBThreshold      =  _gondola_core.events.LTBThreshold       
EventStatus       =  _gondola_core.events.EventStatus        
DataType          =  _gondola_core.events.DataType           

